/* MARS Network Monitoring Engine
   Copyright (C) 1999 Brian H. Trammell
   Copyright (C) 2002 Leapfrog Research & Development, LLC

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, it is available at 
	http:///www.gnu.org/copyleft/gpl.html, or by writing to the
	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

package org.altara.mars.engine;

import org.altara.util.*;
import org.altara.mars.*;
import java.util.*;

/** This class controls the MARS monitoring engine. It manages the monitoring
	work queue, the monitoring threads, and the service status change list.
*/

public class Controller {

	private static final int DEFAULT_WC = 1;	// unithread by default

	private static Controller current;

	public static Controller getCurrent() {
		return current;
	}

	public static void initialize() {
		if (current == null) {
			current = new Controller();
		}
	}

	private MarsModel model;
	private Queue runQueue;
	private HashSet statListeners;
	private HashSet probeListeners;
	private ProbeWorker[] workers;
	private int workerCount;
	private boolean active;

	private Controller() {
		model = null;
		runQueue = new Queue();
		statListeners = new HashSet();
		probeListeners = new HashSet();
		workerCount = DEFAULT_WC;
		active = false;
	}

	public synchronized void start() {
		if (active) {
			// we're already started, so fail silently.
			return;
		}

		if (model == null) {
			// no model means don't start
			throw new RuntimeException("Can't start with null model");
		}

		// build the run queue
		buildRunQueue();

		// create and start a static thread pool
		workers = new ProbeWorker[workerCount];
		for (int i = 0; i < workerCount; i++) {
			workers[i] = new ProbeWorker(runQueue);
			workers[i].start();
		}

		// signify successful start
		this.active = true;
	}

	public synchronized void stop() {
		if (!active) {
			// we're already stopped, so fail silently.
			return;
		}

		// shut down each thread
		for (int i = 0; i < workerCount; i++) {
			workers[i].kill();
		}

		// signify successful stop
		this.active = false;
	}

	public boolean isActive() {
		return active;
	}

	public MarsModel getModel() {
		return model;
	}

	public synchronized void setModel(MarsModel model) {
		if (active) {
			// programming error - must be stopped before setModel.
			throw new RuntimeException("Can't change model while running");
		}
		this.model = model;
	}

	public synchronized void setWorkerCount(int workerCount) {
		this.workerCount = workerCount;
	}

	public void addStatusChangeListener(StatusChangeListener listener) {
		statListeners.add(listener);
	}

	public void removeStatusChangeListener(StatusChangeListener listener) {
		statListeners.remove(listener);
	}

	public void addProbeListener(ProbeListener listener) {
		probeListeners.add(listener);
	}

	public void removeProbeListener(ProbeListener listener) {
		probeListeners.remove(listener);
	}

	public void statusChanged(Service service,
			Status oldStatus, Status newStatus) {
		if (service.getHost().getModel() != model) {
			// ignore services not attached to our current model
			return;
		}

		// construct a StatusChangedEvent to send to all our listeners
		StatusChangeEvent ev =
			new StatusChangeEvent(service,oldStatus,newStatus);
		// send the event
		Iterator listeners = statListeners.iterator();
		while (listeners.hasNext()) {
			((StatusChangeListener)listeners.next()).statusChanged(ev);
		}
	}

	public void probeRun(Service service, Status newStatus) {
		if (service.getHost().getModel() != model) {
			// ignore services not attached to our current model
			return;
		}

		// construct a ProbeEvent to sent to all our listeners
		ProbeEvent ev = new ProbeEvent(service, newStatus);
		// send the event
		Iterator listeners = probeListeners.iterator();
		while (listeners.hasNext()) {
			((ProbeListener)listeners.next()).probeRun(ev);
		}
	}

	private void buildRunQueue() {
		// start by clearing the existing queue
		runQueue = new Queue();

		// run through the tree, depth first.
		Iterator hostNames = model.getHostNames();
		while (hostNames.hasNext()) {
			Host host = model.getHost(((String)hostNames.next()));
			Iterator serviceNames = host.getServiceNames();
			while (serviceNames.hasNext()) {
				Service service =
					host.getService(((String)serviceNames.next()));
				runQueue.enqueue(service.getProbe());
			}
		}
	}
}
